==> esup-canal-intranet-1.20-RC-1.zip: source/org/esupportail/portal/channels/CIntranet/storage/BalancedFileSystemStorage.java <==
package org.esupportail.portal.channels.CIntranet.storage;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.util.Hashtable;

import org.apache.log4j.Priority;
import org.esupportail.portal.channels.CIntranet.beans.Document;
import org.esupportail.portal.channels.CIntranet.beans.Intranet;
import org.jasig.portal.services.LogService;

/**
 * BalancedFileSystemStorage<br>
 * <br>
 * Implmentation d'un IStorageControler avec un stockage des documents sur<br>
 * un systme de fichier physique local (ventuellement partag)<br>
 * Chaque intranet se voit attribuer un dossier dans lequel des dossiers sont crs<br>
 * afin qu'aucun dossier ne contienne plus d'un nombre dtermin de documents<br>
 * <br>
 * (c)Copyright <a href="http://www.esup-portail.org">ESup-Portail 2004</a><br>
 * @author <a href="mailto:mathieu.larchet@univ-nancy2.fr">Mathieu Larchet</a>
 * @version 2.0
 * 
 */

public class BalancedFileSystemStorage implements IStorageControler {

	private Priority nivlog = Priority.DEBUG;
	private String root = null;
	private int size = 100;
	
	/**
	 * Constructeur
	 */
	public BalancedFileSystemStorage() {
	}
	
	/**
	 * Sauvegarde d'un document
	 * @param intranet L'intranet auquel appartient le document
	 * @param doc Le document
	 * @param in Le flux de lecture vers le contenu du document
	 * @throws StorageException
	 */
	public void saveDocument(String intranet, Document doc, InputStream in) throws StorageException {
		LogService.log(nivlog, "BalancedFileSystemStorage::saveDocument()");
		// Vrification paramtres
		if(root == null) {
			LogService.log(Priority.ERROR, "BalancedFileSystemStorage::saveDocument() : root = null");
			throw new StorageException("Le r\u00E9pertoire de base n'est pas correctement configur\u00E9");
		}
		
		// Vrification rpertoire
		File folder = new File(root + "/" + intranet + "/" + getFolder(doc.getId()));
		if(!folder.exists()) {
			folder.mkdir();
		}
		
		// Cration du fichier
		File file = new File(root + "/" + intranet + "/" + getFolder(doc.getId()) + "/" + doc.getId());
		
		// Vrification existence du fichier
		if(file.exists()) {
			LogService.log(Priority.ERROR, "BalancedFileSystemStorage::saveDocument() : Le document " + doc.getId() + " existe d\u00E9j\u00E0");
			throw new StorageException("Erreur lors de la sauvegarde du document");
		}
		try {
			// Cration du fichier vide
			if(!file.createNewFile())
			{
				LogService.log(Priority.ERROR, "BalancedFileSystemStorage::saveDocument() : Erreur lors de la cr\u00E9ation du fichier");
				throw new StorageException("Erreur lors de la sauvegarde du document");
			}
		}
		catch(IOException e) {
			LogService.log(Priority.ERROR, "BalancedFileSystemStorage::saveDocument() : Erreur lors de la cr\u00E9ation du fichier");
			throw new StorageException("Erreur lors de la sauvegarde du document");
		}
		
		// Ecriture des donnes
		try {
			OutputStream out = new FileOutputStream(file);
			
			int size = 0;
            byte[] contentBytes = new byte[8192];
            while ((size = in.read(contentBytes)) != -1) {
                out.write(contentBytes,0, size);
            }
			in.close();
			out.flush();
			out.close();
		}
		catch(FileNotFoundException e) {
			LogService.log(Priority.ERROR, "BalancedFileSystemStorage::saveDocument() : Erreur lors de l'\u00E9criture du fichier");
			throw new StorageException("Erreur lors de la sauvegarde du document");
		}
		catch(IOException e) {
			LogService.log(Priority.ERROR, "BalancedFileSystemStorage::saveDocument() : Erreur lors de l'\u00E9criture du fichier");
			throw new StorageException("Erreur lors de la sauvegarde du document");
		}
	}

	/**
	 * Lecture d'un document
	 * @param intranet L'intranet auquel appartient le document
	 * @param doc Le document
	 * @return Le flux de lecture vers le document stock
	 * @throws StorageException
	 */
	public InputStream loadDocument(String intranet, Document doc) throws StorageException {
		LogService.log(nivlog, "BalancedFileSystemStorage::loadDocument()");
		// Vrification paramtres
		if(root == null) {
			LogService.log(Priority.ERROR, "BalancedFileSystemStorage::loadDocument() : root = null");
			throw new StorageException("Le r\u00E9pertoire de base n'est pas correctement configur\u00E9");
		}
		
		// Rcupration du fichier
		File loadDoc = new File(root + "/" + intranet + "/" + getFolder(doc.getId()) + "/" + doc.getId());
				
		try {
			return new FileInputStream(loadDoc);
		}
		catch(FileNotFoundException e) {
			LogService.log(Priority.ERROR, "BalancedFileSystemStorage::loadDocument() : Erreur lors de la lecture du document " + doc.getId());
			throw new StorageException("Erreur lors de la lecture du document");
		}
	}

	/**
	 * Mise  jour d'un document
	 * @param intranet L'intranet auquel appartient le document
	 * @param doc Le document
	 * @param in Le flux de lecture vers le nouveau contenu, null si pas de mise  jour du contenu
	 * @throws StorageException
	 */
	public synchronized void updateDocument(String intranet, Document doc, InputStream in) throws StorageException {
		LogService.log(nivlog, "BalancedFileSystemStorage::updateDocument()");
		// Vrification paramtres
		if(root == null) {
			LogService.log(Priority.ERROR, "BalancedFileSystemStorage::updateDocument() : root = null");
			throw new StorageException("Le r\u00E9pertoire de base n'est pas correctement configur\u00E9");
		}
		
		// Document non valide
		if(!doc.isValid()) {
			if(in == null) {
				// Recopie du fichier valide
				File valid = new File(root + "/" + intranet + "/" + getFolder(doc.getId()) + "/v" + doc.getId().substring(1));
				File newfile = new File(root + "/" + intranet + "/" + getFolder(doc.getId()) + "/" + doc.getId());
				try {
					// Cration du fichier vide
					if(!newfile.createNewFile())
					{
						LogService.log(Priority.ERROR, "BalancedFileSystemStorage::updateDocument() : Erreur lors de la cr\u00E9ation du fichier");
						throw new StorageException("Erreur lors de la sauvegarde du document");
					}
					InputStream input = new FileInputStream(valid);
					OutputStream out = new FileOutputStream(newfile);
					int size = 0;
		            byte[] contentBytes = new byte[8192];
		            while((size = input.read(contentBytes)) != -1) {
		                out.write(contentBytes,0, size);
		            }
					input.close();
					out.flush();
					out.close();
					return;
				}
				catch(IOException e) {
					LogService.log(Priority.ERROR, "BalancedFileSystemStorage::updateDocument() : Erreur lors de la cr\u00E9ation du fichier");
					throw new StorageException("Erreur lors de la sauvegarde du document");
				}
			}
			else {
				// Ecriture du nouveau fichier
				File newfile = new File(root + "/" + intranet + "/" + getFolder(doc.getId()) + "/" + doc.getId());
				try {
					// Cration du fichier vide
					if(!newfile.createNewFile())
					{
						LogService.log(Priority.ERROR, "BalancedFileSystemStorage::updateDocument() : Erreur lors de la cr\u00E9ation du fichier");
						throw new StorageException("Erreur lors de la sauvegarde du document");
					}
					OutputStream out = new FileOutputStream(newfile);
					int size = 0;
		            byte[] contentBytes = new byte[8192];
		            while((size = in.read(contentBytes)) != -1) {
		                out.write(contentBytes,0, size);
		            }
					in.close();
					out.flush();
					out.close();
					return;
				}
				catch(IOException e) {
					LogService.log(Priority.ERROR, "BalancedFileSystemStorage::updateDocument() : Erreur lors de la cr\u00E9ation du fichier");
					throw new StorageException("Erreur lors de la sauvegarde du document");
				}
			}
		}
		
		// Document valide
		File valid = new File(root + "/" + intranet + "/" + getFolder(doc.getId()) + "/" + doc.getId());
		valid.delete();
		try {
			// Cration du fichier vide
			if(!valid.createNewFile())
			{
				LogService.log(Priority.ERROR, "BalancedFileSystemStorage::updateDocument() : Erreur lors de la cr\u00E9ation du fichier");
				throw new StorageException("Erreur lors de la sauvegarde du document");
			}
			OutputStream out = new FileOutputStream(valid);
			int size = 0;
            byte[] contentBytes = new byte[8192];
            while((size = in.read(contentBytes)) != -1) {
                out.write(contentBytes,0, size);
            }
			in.close();
			out.flush();
			out.close();
		}
		catch(IOException e) {
			LogService.log(Priority.ERROR, "BalancedFileSystemStorage::updateDocument() : Erreur lors de la cr\u00E9ation du fichier");
			throw new StorageException("Erreur lors de la sauvegarde du document");
		}
	}
	
	/**
	 * Suppression d'un document
	 * @param intranet L'intranet auquel appartient le document
	 * @param doc Le document
	 * @throws StorageException
	 */
	public synchronized void destroyDocument(String intranet, Document doc) throws StorageException {
		LogService.log(nivlog, "BalancedFileSystemStorage::destroyDocument()");
		// Vrification paramtres
		if(root == null) {
			LogService.log(Priority.ERROR, "BalancedFileSystemStorage::destroyDocument() : root = null");
			throw new StorageException("Le r\u00E9pertoire de base n'est pas correctement configur\u00E9");
		}
		
		// Rcupration du fichier
		File file = new File(root + "/" + intranet + "/" + getFolder(doc.getId()) + "/" + doc.getId());
				
		// Suppression du fichier
		if(!file.delete()) {
			LogService.log(Priority.ERROR, "BalancedFileSystemStorage::destroyDocument() : Erreur lors de la suppression du fichier " + doc.getId());
			LogService.log(Priority.FATAL, "BalancedFileSystemStorage::destroyDocument() : Un document supprim\u00E9 de la base est encore stock\u00E9");
			throw new StorageException("Erreur lors de la suppression du document");
		}
	}

	/**
	 * Validation d'un document
	 * @param intranet L'intranet auquel appartient le document
	 * @param doc Le document  valider
	 * @throws StorageException
	 */
	public synchronized void validDocument(String intranet, Document doc) throws StorageException {
		LogService.log(nivlog, "BalancedFileSystemStorage::validDocument()");
		// Vrification paramtres
		if(root == null) {
			LogService.log(Priority.ERROR, "BalancedFileSystemStorage::validDocument() : root = null");
			throw new StorageException("Le r\u00E9pertoire de base n'est pas correctement configur\u00E9");
		}
		
		// Cration des fichiers
		File validfile = new File(root + "/" + intranet + "/" + getFolder(doc.getId()) + "/" + doc.getId());
		File oldfile = new File(root + "/" + intranet + "/" + getFolder(doc.getId()) + "/i" + doc.getId().substring(1));
		
		// Eventuellement suppression de l'ancien fichier et cration du nouveau
		validfile.delete();
		if(!oldfile.renameTo(validfile)) {
			LogService.log(Priority.ERROR, "BalancedFileSystemStorage::validDocument() : Erreur lors de la validation du fichier " + doc.getId());
			LogService.log(Priority.FATAL, "BalancedFileSystemStorage::validDocument() : La validation a \u00E9chou\u00E9 le fichier non valid\u00E9 est encore pr\u00E9sent");
			throw new StorageException("Erreur lors de la validation du document");
		}
	}
	
	/**
	 * Invalidation d'un document
	 * @param intranet L'intranet auquel appartient le document
	 * @param doc Le document  invalider
	 * @throws StorageException
	 */
	public synchronized void invalidDocument(String intranet, Document doc) throws StorageException {
	    LogService.log(nivlog, "BalancedFileSystemStorage::invalidDocument()");
		// Vrification paramtres
		if(root == null) {
			LogService.log(Priority.ERROR, "BalancedFileSystemStorage::invalidDocument() : root = null");
			throw new StorageException("Le r\u00E9pertoire de base n'est pas correctement configur\u00E9");
		}
		
		// Cration des fichiers
		File invalidfile = new File(root + "/" + intranet + "/" + getFolder(doc.getId()) + "/" + doc.getId());
		File oldfile = new File(root + "/" + intranet + "/" + getFolder(doc.getId()) + "/v" + doc.getId().substring(1));
		
		if(!oldfile.renameTo(invalidfile)) {
			LogService.log(Priority.ERROR, "BalancedFileSystemStorage::invalidDocument() : Erreur lors de l'invalidation du fichier " + doc.getId());
			LogService.log(Priority.FATAL, "BalancedFileSystemStorage::invalidDocument() : L'invalidation a \u00Echou\u00E9 le fichier valid\u00E9 est encore pr\u00E9sent");
			throw new StorageException("Erreur lors de l'invalidation du document");
		}
	}
	
	/**
	 * Cration d'un intranet
	 * @param intranet L'intranet  crer
	 * @throws StorageException
	 */
	public void createIntranet(Intranet intranet) throws StorageException {
		LogService.log(nivlog, "BalancedFileSystemStorage::createIntranet()");
		// Vrification paramtres
		if(root == null) {
			LogService.log(Priority.ERROR, "BalancedFileSystemStorage::createIntranet() : root = null");
			throw new StorageException("Le r\u00E9pertoire de base n'est pas correctement configur\u00E9");
		}
		
		// Cration du dossier racine
		File newFold = new File(root + "/" + intranet.getId());
		if(newFold.exists()) {
			LogService.log(Priority.ERROR, "BalancedFileSystemStorage::createIntranet() : Le dossier " + intranet.getId() + " existe d\u00E9j\u00E0");
			throw new StorageException("Erreur lors de la cr\u00E9ation de l'intranet");
		}
		if(!newFold.mkdir()) {
			LogService.log(Priority.ERROR, "BalancedFileSystemStorage::createIntranet() : Erreur lors de la cr\u00E9ation du dossier " + intranet.getId());
			throw new StorageException("Erreur lors de la cr\u00E9ation de l'intranet");
		}
	}

	/**
	 * Suppression d'un intranet et de tous ses documents
	 * @param intranet L'intranet  supprimer
	 * @throws StorageException
	 */
	public synchronized void destroyIntranet(String intranet) throws StorageException {
		LogService.log(nivlog, "BalancedFileSystemStorage::destroyIntranet()");
		if(root == null) {
			LogService.log(Priority.ERROR, "BalancedFileSystemStorage::destroyIntranet() : root = null");
			throw new StorageException("Le rpertoire de base n'est pas correctement configur\u00E9");
		}
		LogService.log(Priority.FATAL, "BalancedFileSystemStorage::destroyIntranet() : La suppression de l'intranet " + intranet + " doit \u00EAtre r\u00E9alise manuellement");
	}
	
	
	/**
	 * Calcule le rpertoire o lire / crire un document<br>
	 * en fonction de son identifiant
	 * @param id L'identifiant du document
	 * @return Le chemin o lire / crire le document
	 */
	private String getFolder(String id) {
		int number = new Integer(id.substring(1)).intValue();
		return new Integer(number / size).toString();
	}
	
	/**
	 * Liste des paramtres spcifiques  l'implmentation du IStorageControler<br>
	 * Ici deux paramtres obligatoires :<br>
	 * - size qui contient le nombre maximal de documents par dossier<br>
	 * - path qui indique le chemin o crer les intranets
	 * @param parameters
	 */
	public void setParameters(Hashtable parameters) {
		LogService.log(nivlog, "BalancedFileSystemStorage::setParameters()");
		root = (String)parameters.get("path");
		size = new Integer((String)parameters.get("size")).intValue();
	}
}

==> esup-canal-intranet-2.00-RC-1.zip: source/org/esupportail/portal/channels/CIntranet/storage/BalancedFileSystemStorage.java <==
package org.esupportail.portal.channels.CIntranet.storage;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.util.Hashtable;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.esupportail.portal.channels.CIntranet.beans.Document;
import org.esupportail.portal.channels.CIntranet.beans.Intranet;

/**
 * BalancedFileSystemStorage<br>
 * <br>
 * Implmentation d'un IStorageControler avec un stockage des documents sur<br>
 * un systme de fichier physique local (ventuellement partag)<br>
 * Chaque intranet se voit attribuer un dossier dans lequel des dossiers sont crs<br>
 * afin qu'aucun dossier ne contienne plus d'un nombre dtermin de documents<br>
 * <br>
 * (c)Copyright <a href="http://www.esup-portail.org">ESup-Portail 2004</a><br>
 * @author <a href="mailto:mathieu.larchet@univ-nancy2.fr">Mathieu Larchet</a>
 * @version 2.0
 * 
 */

public class BalancedFileSystemStorage implements IStorageControler {

    protected static final Log log = LogFactory.getLog(BalancedFileSystemStorage.class);
	private String root = null;
	private int size = 100;
	
	/**
	 * Constructeur
	 */
	public BalancedFileSystemStorage() {
	}
	
	/**
	 * Sauvegarde d'un document
	 * @param intranet L'intranet auquel appartient le document
	 * @param doc Le document
	 * @param in Le flux de lecture vers le contenu du document
	 * @throws StorageException
	 */
	public synchronized void saveDocument(String intranet, Document doc, InputStream in) throws StorageException {
		if(log.isDebugEnabled()) {
		    log.debug("BalancedFileSystemStorage::saveDocument()");
		}
		// Vrification paramtres
		if(root == null) {
			log.error("BalancedFileSystemStorage::saveDocument() : root = null");
			throw new StorageException("Le r\u00E9pertoire de base n'est pas correctement configur\u00E9");
		}
		
		// Vrification rpertoire
		File folder = new File(root + "/" + intranet + "/" + getFolder(doc.getId()));
		if(!folder.exists()) {
			folder.mkdir();
		}
		
		// Cration du fichier
		File file = new File(root + "/" + intranet + "/" + getFolder(doc.getId()) + "/" + doc.getId());
		
		// Vrification existence du fichier
		if(file.exists()) {
			log.error("BalancedFileSystemStorage::saveDocument() : Le document " + doc.getId() + " existe d\u00E9j\u00E0");
			throw new StorageException("Erreur lors de la sauvegarde du document");
		}
		try {
			// Cration du fichier vide
			if(!file.createNewFile())
			{
				log.error("BalancedFileSystemStorage::saveDocument() : Erreur lors de la cr\u00E9ation du fichier");
				throw new StorageException("Erreur lors de la sauvegarde du document");
			}
		}
		catch(IOException e) {
			log.error("BalancedFileSystemStorage::saveDocument() : Erreur lors de la cr\u00E9ation du fichier");
			throw new StorageException("Erreur lors de la sauvegarde du document");
		}
		
		// Ecriture des donnes
		try {
			OutputStream out = new FileOutputStream(file);
			
			int size = 0;
            byte[] contentBytes = new byte[8192];
            while ((size = in.read(contentBytes)) != -1) {
                out.write(contentBytes,0, size);
            }
			in.close();
			out.flush();
			out.close();
		}
		catch(FileNotFoundException e) {
			log.error("BalancedFileSystemStorage::saveDocument() : Erreur lors de l'\u00E9criture du fichier");
			throw new StorageException("Erreur lors de la sauvegarde du document");
		}
		catch(IOException e) {
			log.error("BalancedFileSystemStorage::saveDocument() : Erreur lors de l'\u00E9criture du fichier");
			throw new StorageException("Erreur lors de la sauvegarde du document");
		}
	}

	/**
	 * Lecture d'un document
	 * @param intranet L'intranet auquel appartient le document
	 * @param doc Le document
	 * @return Le flux de lecture vers le document stock
	 * @throws StorageException
	 */
	public synchronized InputStream loadDocument(String intranet, Document doc) throws StorageException {
	    if(log.isDebugEnabled()) {
		    log.debug( "BalancedFileSystemStorage::loadDocument()");
	    }
		// Vrification paramtres
		if(root == null) {
			log.error("BalancedFileSystemStorage::loadDocument() : root = null");
			throw new StorageException("Le r\u00E9pertoire de base n'est pas correctement configur\u00E9");
		}
		
		// Rcupration du fichier
		File loadDoc = new File(root + "/" + intranet + "/" + getFolder(doc.getId()) + "/" + doc.getId());
				
		try {
			return new FileInputStream(loadDoc);
		}
		catch(FileNotFoundException e) {
			log.error("BalancedFileSystemStorage::loadDocument() : Erreur lors de la lecture du document " + doc.getId());
			throw new StorageException("Erreur lors de la lecture du document");
		}
	}

	/**
	 * Mise  jour d'un document
	 * @param intranet L'intranet auquel appartient le document
	 * @param doc Le document
	 * @param in Le flux de lecture vers le nouveau contenu, null si pas de mise  jour du contenu
	 * @throws StorageException
	 */
	public synchronized void updateDocument(String intranet, Document doc, InputStream in) throws StorageException {
	    if(log.isDebugEnabled()) {
		    log.debug("BalancedFileSystemStorage::updateDocument()");
	    }
		// Vrification paramtres
		if(root == null) {
			log.error("BalancedFileSystemStorage::updateDocument() : root = null");
			throw new StorageException("Le r\u00E9pertoire de base n'est pas correctement configur\u00E9");
		}
		
		// Document non valide
		if(!doc.isValid()) {
			if(in == null) {
				// Recopie du fichier valide
				File valid = new File(root + "/" + intranet + "/" + getFolder(doc.getId()) + "/v" + doc.getId().substring(1));
				File newfile = new File(root + "/" + intranet + "/" + getFolder(doc.getId()) + "/" + doc.getId());
				try {
					// Cration du fichier vide
					if(!newfile.createNewFile())
					{
						log.error("BalancedFileSystemStorage::updateDocument() : Erreur lors de la cr\u00E9ation du fichier");
						throw new StorageException("Erreur lors de la sauvegarde du document");
					}
					InputStream input = new FileInputStream(valid);
					OutputStream out = new FileOutputStream(newfile);
					int size = 0;
		            byte[] contentBytes = new byte[8192];
		            while((size = input.read(contentBytes)) != -1) {
		                out.write(contentBytes,0, size);
		            }
					input.close();
					out.flush();
					out.close();
					return;
				}
				catch(IOException e) {
					log.error("BalancedFileSystemStorage::updateDocument() : Erreur lors de la cr\u00E9ation du fichier");
					throw new StorageException("Erreur lors de la sauvegarde du document");
				}
			}
			else {
				// Ecriture du nouveau fichier
				File newfile = new File(root + "/" + intranet + "/" + getFolder(doc.getId()) + "/" + doc.getId());
				try {
					// Cration du fichier vide
					if(!newfile.createNewFile())
					{
						log.error("BalancedFileSystemStorage::updateDocument() : Erreur lors de la cr\u00E9ation du fichier");
						throw new StorageException("Erreur lors de la sauvegarde du document");
					}
					OutputStream out = new FileOutputStream(newfile);
					int size = 0;
		            byte[] contentBytes = new byte[8192];
		            while((size = in.read(contentBytes)) != -1) {
		                out.write(contentBytes,0, size);
		            }
					in.close();
					out.flush();
					out.close();
					return;
				}
				catch(IOException e) {
					log.error("BalancedFileSystemStorage::updateDocument() : Erreur lors de la cr\u00E9ation du fichier");
					throw new StorageException("Erreur lors de la sauvegarde du document");
				}
			}
		}
		
		// Document valide
		File valid = new File(root + "/" + intranet + "/" + getFolder(doc.getId()) + "/" + doc.getId());
		valid.delete();
		try {
			// Cration du fichier vide
			if(!valid.createNewFile())
			{
				log.error("BalancedFileSystemStorage::updateDocument() : Erreur lors de la cr\u00E9ation du fichier");
				throw new StorageException("Erreur lors de la sauvegarde du document");
			}
			OutputStream out = new FileOutputStream(valid);
			int size = 0;
            byte[] contentBytes = new byte[8192];
            while((size = in.read(contentBytes)) != -1) {
                out.write(contentBytes,0, size);
            }
			in.close();
			out.flush();
			out.close();
		}
		catch(IOException e) {
			log.error("BalancedFileSystemStorage::updateDocument() : Erreur lors de la cr\u00E9ation du fichier");
			throw new StorageException("Erreur lors de la sauvegarde du document");
		}
	}
	
	/**
	 * Suppression d'un document
	 * @param intranet L'intranet auquel appartient le document
	 * @param doc Le document
	 * @throws StorageException
	 */
	public synchronized void destroyDocument(String intranet, Document doc) throws StorageException {
	    if(log.isDebugEnabled()) {
		    log.debug("BalancedFileSystemStorage::destroyDocument()");
	    }
		// Vrification paramtres
		if(root == null) {
			log.error("BalancedFileSystemStorage::destroyDocument() : root = null");
			throw new StorageException("Le r\u00E9pertoire de base n'est pas correctement configur\u00E9");
		}
		
		// Rcupration du fichier
		File file = new File(root + "/" + intranet + "/" + getFolder(doc.getId()) + "/" + doc.getId());
				
		// Suppression du fichier
		if(!file.delete()) {
			log.error("BalancedFileSystemStorage::destroyDocument() : Erreur lors de la suppression du fichier " + doc.getId());
			log.fatal("BalancedFileSystemStorage::destroyDocument() : Un document supprim\u00E9 de la base est encore stock\u00E9");
			throw new StorageException("Erreur lors de la suppression du document");
		}
	}

	/**
	 * Validation d'un document
	 * @param intranet L'intranet auquel appartient le document
	 * @param doc Le document  valider
	 * @throws StorageException
	 */
	public synchronized void validDocument(String intranet, Document doc) throws StorageException {
	    if(log.isDebugEnabled()) {
		    log.debug("BalancedFileSystemStorage::validDocument()");
	    }
		// Vrification paramtres
		if(root == null) {
			log.error("BalancedFileSystemStorage::validDocument() : root = null");
			throw new StorageException("Le r\u00E9pertoire de base n'est pas correctement configur\u00E9");
		}
		
		// Cration des fichiers
		File validfile = new File(root + "/" + intranet + "/" + getFolder(doc.getId()) + "/" + doc.getId());
		File oldfile = new File(root + "/" + intranet + "/" + getFolder(doc.getId()) + "/i" + doc.getId().substring(1));
		
		// Eventuellement suppression de l'ancien fichier et cration du nouveau
		validfile.delete();
		if(!oldfile.renameTo(validfile)) {
			log.error("BalancedFileSystemStorage::validDocument() : Erreur lors de la validation du fichier " + doc.getId());
			log.fatal("BalancedFileSystemStorage::validDocument() : La validation a \u00E9chou\u00E9 le fichier non valid\u00E9 est encore pr\u00E9sent");
			throw new StorageException("Erreur lors de la validation du document");
		}
	}
	
	/**
	 * Invalidation d'un document
	 * @param intranet L'intranet auquel appartient le document
	 * @param doc Le document  invalider
	 * @throws StorageException
	 */
	public synchronized void invalidDocument(String intranet, Document doc) throws StorageException {
	    if(log.isDebugEnabled()) {
		    log.debug("BalancedFileSystemStorage::invalidDocument()");
	    }
		// Vrification paramtres
		if(root == null) {
			log.error("BalancedFileSystemStorage::invalidDocument() : root = null");
			throw new StorageException("Le r\u00E9pertoire de base n'est pas correctement configur\u00E9");
		}
		
		// Cration des fichiers
		File invalidfile = new File(root + "/" + intranet + "/" + getFolder(doc.getId()) + "/" + doc.getId());
		File oldfile = new File(root + "/" + intranet + "/" + getFolder(doc.getId()) + "/v" + doc.getId().substring(1));
		
		if(!oldfile.renameTo(invalidfile)) {
			log.error("BalancedFileSystemStorage::invalidDocument() : Erreur lors de l'invalidation du fichier " + doc.getId());
			log.fatal("BalancedFileSystemStorage::invalidDocument() : L'invalidation a \u00Echou\u00E9 le fichier valid\u00E9 est encore pr\u00E9sent");
			throw new StorageException("Erreur lors de l'invalidation du document");
		}
	}
	
	/**
	 * Cration d'un intranet
	 * @param intranet L'intranet  crer
	 * @throws StorageException
	 */
	public void createIntranet(Intranet intranet) throws StorageException {
	    if(log.isDebugEnabled()) {
		    log.debug("BalancedFileSystemStorage::createIntranet()");
	    }
		// Vrification paramtres
		if(root == null) {
			log.error("BalancedFileSystemStorage::createIntranet() : root = null");
			throw new StorageException("Le r\u00E9pertoire de base n'est pas correctement configur\u00E9");
		}
		
		// Cration du dossier racine
		File newFold = new File(root + "/" + intranet.getId());
		if(newFold.exists()) {
			log.error("BalancedFileSystemStorage::createIntranet() : Le dossier " + intranet.getId() + " existe d\u00E9j\u00E0");
			throw new StorageException("Erreur lors de la cr\u00E9ation de l'intranet");
		}
		if(!newFold.mkdir()) {
			log.error("BalancedFileSystemStorage::createIntranet() : Erreur lors de la cr\u00E9ation du dossier " + intranet.getId());
			throw new StorageException("Erreur lors de la cr\u00E9ation de l'intranet");
		}
	}

	/**
	 * Suppression d'un intranet et de tous ses documents
	 * @param intranet L'intranet  supprimer
	 * @throws StorageException
	 */
	public synchronized void destroyIntranet(String intranet) throws StorageException {
	    if(log.isDebugEnabled()) {
		    log.debug("BalancedFileSystemStorage::destroyIntranet()");
	    }
		if(root == null) {
			log.error("BalancedFileSystemStorage::destroyIntranet() : root = null");
			throw new StorageException("Le rpertoire de base n'est pas correctement configur\u00E9");
		}
		log.fatal("BalancedFileSystemStorage::destroyIntranet() : La suppression de l'intranet " + intranet + " doit \u00EAtre r\u00E9alise manuellement");
	}
	
	
	/**
	 * Calcule le rpertoire o lire / crire un document<br>
	 * en fonction de son identifiant
	 * @param id L'identifiant du document
	 * @return Le chemin o lire / crire le document
	 */
	private String getFolder(String id) {
		int number = new Integer(id.substring(1)).intValue();
		return new Integer(number / size).toString();
	}
	
	/**
	 * Liste des paramtres spcifiques  l'implmentation du IStorageControler<br>
	 * Ici deux paramtres obligatoires :<br>
	 * - size qui contient le nombre maximal de documents par dossier<br>
	 * - path qui indique le chemin o crer les intranets
	 * @param parameters
	 */
	public void setParameters(Hashtable parameters) {
	    if(log.isDebugEnabled()) {
		    log.debug("BalancedFileSystemStorage::setParameters()");
	    }
		root = (String)parameters.get("path");
		size = new Integer((String)parameters.get("size")).intValue();
	}
}
